import os

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import Q

from datawok.conf import settings

from .meta_category import MetaCategory
from .get_models import models as datawok_models


class Meta(models.Model):
    """
    Meta information about your other models.
    """

    datawok_model = models.OneToOneField(
        ContentType,
        on_delete=models.CASCADE,
        limit_choices_to=Q(
            model__in=[model.__name__.lower() for model in datawok_models]
        ),
    )
    category = models.ForeignKey(MetaCategory, on_delete=models.PROTECT)

    owner = models.ForeignKey(User, on_delete=models.PROTECT)

    title = models.CharField(max_length=50)
    description = models.CharField(max_length=500, null=True, blank=True)
    query_alias = models.CharField(max_length=50)
    publish_path = models.CharField(max_length=500)

    created_date = models.DateTimeField(auto_now_add=True)
    last_published = models.DateTimeField(null=True, blank=True)

    def get_publish_path(self):
        date_path = self.created_date.strftime("%Y%m%d")
        publish_path = self.publish_path

        if publish_path[0] == "/":
            publish_path = publish_path[1:]

        if publish_path[-1] == "/":
            publish_path = publish_path[:-1]

        return os.path.join(settings.S3_UPLOAD_ROOT, publish_path, date_path)

    def get_publish_link(self):
        publish_path = self.get_publish_path()

        if publish_path[0] == "/":
            publish_path = publish_path[1:]

        if publish_path[-1] == "/":
            publish_path = publish_path[:-1]

        return os.path.join(
            settings.AWS_PUBLIC_ROOT, publish_path, "data.json"
        )

    def get_model_name(self):
        return self.datawok_model.model

    def __str__(self):
        return self.title if self.title else self.datawok_model.model

    class Meta:
        unique_together = ("category", "query_alias")



from django.db.models import Q
from django.shortcuts import get_object_or_404
from datawok.views.base import Base
from datawok.generators.serializers import generate_serializer
from datawok.utils.get_field_type import get_field_type
from datawok.models import Query
from datawok.serializers import QuerySerializer
from datawok.models import Meta
from datawok.conf import settings


class Detail(Base):
    template_name = "datawok/detail.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        fields = {}
        for field in self.model._meta.get_fields():
            field_type = get_field_type(field)
            fields[str(field.name)] = field_type
        context["fields"] = Base.prep_data_for_injection(fields)

        context["queries"] = Base.prep_data_for_injection(
            Query.objects.filter(
                Q(datawok_model__model=self.model.__name__.lower())
            ),
            QuerySerializer,
            many=True,
        )

        context["model"] = self.model.__name__.lower()

        meta = get_object_or_404(Meta, datawok_model__model=context["model"])
        context["meta_pk"] = meta.pk

        context["query_limit"] = settings.QUERY_LIMIT

        return context


def generate_detail_view(model, serializer=None):
    """
    Dynamically generates a detail view for a given model.
    """
    name = "{}DetailView".format(model.__name__.title())

    if serializer is None:
        serializer = generate_serializer(model)

    return type(name, (Detail,), {"model": model, "serializer": serializer})



from rest_framework.response import Response
from rest_framework.views import APIView
from datawok.models import Meta
from datawok.celery import publish_to_aws
from datawok.utils.api_auth import TokenAPIAuthentication


class Publish(APIView):
    """
    View to handle publishing to AWS
    """

    def post(self, request, format=None):
        """
        Publish a model's data.
        """
        model_name = self.model.__name__.lower()

        try:
            Meta.objects.get(datawok_model__model=model_name)
        except Meta.DoesNotExist:
            return Response(
                "No Meta instance for model: {}.".format(model_name), 404
            )

        publish_to_aws.delay(model_name)

        return Response("OK", 200)


def generate_publish_view(model):
    """
    Dynamically generates a publish view for a given model.
    """
    name = "{}PublishView".format(model.__name__.title())

    return type(
        name,
        (Publish,),
        {
            "model": model,
            "authentication_classes": (TokenAPIAuthentication,),
            "permission_classes": [],
        },
    )



import psycopg2
from django.conf import settings as project_settings
from rest_framework.response import Response
from rest_framework.views import APIView

from datawok.conf import settings
from datawok.utils.api_auth import TokenAPIAuthentication

database = project_settings.DATABASES[settings.DATABASE]


class SQL(APIView):
    """
    View to process raw read-only SQL queries.
    """

    authentication_classes = (TokenAPIAuthentication,)
    permission_classes = ()

    def post(self, request, format=None):
        """
        Execute a read-only SQL query.
        """
        query = request.data["query"]

        cnn = psycopg2.connect(
            dbname=database["NAME"],
            user=database["USER"],
            password=database["PASSWORD"],
            host=database["HOST"],
            port=database["PORT"],
            options="-c statement_timeout={}".format(settings.QUERY_TIMEOUT),
        )
        cnn.set_session(readonly=True)
        cur = cnn.cursor()

        try:
            cur.execute(query)
        except Exception as e:
            cnn.close()
            return Response("Database query error: {}".format(e).strip(), 500)

        resp = cur.fetchall()
        columns = [desc[0] for desc in cur.description]

        output = []
        for rowi, row in enumerate(resp):
            if rowi >= settings.QUERY_LIMIT:
                break

            entry = {}
            for index, column in enumerate(columns):
                entry[column] = row[index]
            output.append(entry)

        cnn.close()

        return Response(output, 200)



from django.db import models

MODEL = "model"
SERIALIZER = "serializer"
PUBLISH_SERIALIZER = "publish_serializer"
VIEWSET = "viewset"
DETAIL_VIEW = "detail"
PUBLISH_VIEW = "publish"
UPLOAD_CSV_FILE_VIEW = "upload_csv_file"

STRING_FIELDS = [
    models.CharField,
    models.EmailField,
    models.FilePathField,
    models.GenericIPAddressField,
    models.SlugField,
    models.TextField,
    models.URLField,
    models.UUIDField,
]

DATE_FIELDS = [models.DateField, models.DateTimeField, models.TimeField]

INTEGER_FIELDS = [
    models.AutoField,
    models.BigAutoField,
    models.BigIntegerField,
    models.IntegerField,
    models.PositiveIntegerField,
    models.PositiveSmallIntegerField,
    models.SmallIntegerField,
]

FLOAT_FIELDS = [models.DecimalField, models.FloatField]

BOOLEAN_FIELDS = [models.BooleanField, models.NullBooleanField]

FK_FIELDS = [models.ForeignKey, models.OneToOneField, models.ManyToManyField]

UNSUPPORTED_FIELDS = [
    models.BinaryField,
    models.DurationField,
    models.FileField,
    models.ImageField,
]



from django.contrib import admin

from datawok.models import Meta, MetaCategory, Query, datawok_models

for model in datawok_models:
    admin.site.register(model)

admin.site.register(Meta)
admin.site.register(MetaCategory)
admin.site.register(Query)



from rest_framework import serializers
from datawok.models import Meta


class MetaSerializer(serializers.ModelSerializer):
    publish_path = serializers.SerializerMethodField()
    publish_link = serializers.SerializerMethodField()
    datawok_model = serializers.SerializerMethodField()
    owner = serializers.SerializerMethodField()
    category = serializers.SerializerMethodField()

    def get_publish_path(self, obj):
        return obj.get_publish_path()

    def get_publish_link(self, obj):
        return obj.get_publish_link()

    def get_datawok_model(self, obj):
        return obj.datawok_model.model

    def get_owner(self, obj):
        return {"id": obj.owner.pk, "name": str(obj.owner)}

    def get_category(self, obj):
        return {"slug": obj.category.slug, "title": str(obj.category)}

    class Meta:
        model = Meta
        fields = "__all__"



import tempfile

from rest_framework.parsers import FileUploadParser
from rest_framework.response import Response
from rest_framework.views import APIView

from datawok.conf import settings
from datawok.utils.api_auth import TokenAPIAuthentication


class UploadCSVFileView(APIView):
    parser_classes = (FileUploadParser,)
    model = None
    media_type = "text/csv"

    def put(self, request, format=None):
        file_obj = request.data["file"]
        try:
            file_data = file_obj.read().decode(settings.CSV_FILE_ENCODING)
        except Exception as e:
            return Response(
                {"msg": "Error reading or decoding CSV", "error": e},
                status=500,
            )
        with tempfile.TemporaryFile(mode="r+") as tmp:
            try:
                tmp.write(file_data)
                tmp.seek(0)
            except Exception as e:
                return Response(
                    {"msg": "Error loading CSV", "error": e}, status=500
                )
            try:
                self.model.objects.from_csv(tmp)
            except Exception as e:
                return Response(
                    {"msg": "Error updating data.", "error": str(e)},
                    status=500,
                )
        return Response({"msg": "success"}, status=200)


def generate_upload_csv_file_view(model):
    """
    Dynamically generates a viewset for a given model.
    """
    name = "{}UploadCSVFileView".format(model.__name__.title())

    return type(
        name,
        (UploadCSVFileView,),
        {
            "model": model,
            "authentication_classes": (TokenAPIAuthentication,),
            "permission_classes": [],
        },
    )



import json
import os
import logging

from celery import shared_task

from datawok.models import datawok_models, Meta
from datawok.serializers import datawok_publish_serializers
from datawok.utils.aws import defaults, get_bucket
from django.utils import timezone

logger = logging.getLogger("tasks")


@shared_task(acks_late=True)
def publish_to_aws(model_name, filename="data.json", data=None):
    model = [
        model
        for model in datawok_models
        if model.__name__.lower() == model_name
    ][0]

    try:
        meta = Meta.objects.get(datawok_model__model=model_name)
    except Meta.DoesNotExist:
        return

    if not data:
        i = datawok_models.index(model)
        serializer = datawok_publish_serializers[i]
        data = json.dumps(serializer(model.objects.all(), many=True).data)

    key = os.path.join(meta.get_publish_path(), filename.lstrip("/"))

    bucket = get_bucket()

    bucket.put_object(
        Key=key,
        ACL=defaults.ACL,
        Body=data,
        CacheControl=defaults.CACHE_HEADER,
        ContentType="application/json",
    )

    logger.info("Published to AWS")

    meta.last_published = timezone.now()
    meta.save()

    logger.info("Updated meta last_published.")



from django.urls import include, path
from rest_framework import routers

from datawok.models import datawok_models
from datawok.views import (
    SQL,
    List,
    upload_csv_file_views,
    publish_views,
    detail_views,
)
from datawok.viewsets import datawok_viewsets, MetaViewset, QueryViewset

router = routers.DefaultRouter()

urlpatterns = [
    path("", List.as_view(), name="datawok-list"),
    path("api/sql/", SQL.as_view(), name="datawok-sql"),
]

router.register(r"meta", MetaViewset)
router.register(r"query", QueryViewset)

for i, model in enumerate(datawok_models):
    name = model.__name__.lower()
    # Model viewsets
    router.register(
        r"{}".format(name),
        datawok_viewsets[i],
        base_name="datawok-{}".format(name),
    )

    # CSV file upload views
    urlpatterns += [
        path(
            r"api/{}/upload/csv/file/".format(name),
            upload_csv_file_views[i].as_view(),
            name="datawok-upload-csv-file-{}".format(name),
        )
    ]

    # Publish views
    urlpatterns += [
        path(
            r"api/{}/publish/".format(name),
            publish_views[i].as_view(),
            name="datawok-publish-{}".format(name),
        )
    ]

    # Detail views
    urlpatterns += [
        path(
            r"{}/".format(name),
            detail_views[i].as_view(),
            name="datawok-detail-{}".format(name),
        )
    ]

urlpatterns += [path("api/", include(router.urls))]



from datawok.conf import settings
from datawok.utils.importers import import_class
from datawok.constants import MODEL

models = []

for module_paths in settings.MODELS:
    model = (
        import_class(module_paths[MODEL])
        if isinstance(module_paths, dict) and MODEL in module_paths
        else import_class(module_paths)
    )

    models.append(model)



from rest_framework.viewsets import ModelViewSet
from rest_framework.decorators import action
from rest_framework.pagination import PageNumberPagination

from datawok.utils.api_auth import TokenAPIAuthentication
from datawok.utils.process_input_data import process_input_data
from rest_framework.response import Response

from .serializers import generate_serializer


class Pagination(PageNumberPagination):
    page_size = 100
    page_size_query_param = "size"


class DatawokModelViewset(ModelViewSet):
    pagination_class = Pagination
    authentication_classes = (TokenAPIAuthentication,)
    permission_classes = []

    @action(detail=False, methods=["post"])
    def bulk_create(self, request, pk=None):
        data = request.data

        try:
            model = self.model
            data = process_input_data(model, data)
            model.objects.bulk_create([model(**datum) for datum in data])
        except Exception as e:
            print(e)
            return Response(
                {"msg": "Error reading or decoding CSV.", "error": str(e)},
                status=500,
            )

        return Response({"msg": "success"}, status=200)


def generate_model_viewset(model, serializer=None):
    """
    Dynamically generates a viewset for a given model.
    """
    name = "{}ViewSet".format(model.__name__.title())

    if serializer is None:
        serializer = generate_serializer(model)

    return type(
        name,
        (DatawokModelViewset,),
        {
            "model": model,
            "serializer_class": serializer,
            "queryset": model.objects.get_queryset().order_by("pk"),
        },
    )



from rest_framework import viewsets
from datawok.models import Meta
from datawok.serializers import MetaSerializer
from datawok.utils.api_auth import TokenAPIAuthentication


class MetaViewset(viewsets.ModelViewSet):
    model = Meta
    authentication_classes = (TokenAPIAuthentication,)
    permission_classes = []
    pagination_class = None
    serializer_class = MetaSerializer
    queryset = Meta.objects.all()



class DatawokRouter:
    """
    A default router you can use to route to a separate datawok database.
    """

    def db_for_read(self, model, **hints):
        if model._meta.app_label == "datawok":
            return "datawok"
        return None

    def db_for_write(self, model, **hints):
        if model._meta.app_label == "datawok":
            return "datawok"
        return None

    def allow_relation(self, obj1, obj2, **hints):
        if (
            obj1._meta.app_label == "datawok"
            or obj2._meta.app_label == "datawok"
        ):
            return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if app_label == "datawok":
            return db == "datawok"
        return None



import boto3
from storages.backends.s3boto3 import S3Boto3Storage

from datawok.conf import settings


def get_bucket():
    session = boto3.session.Session(
        region_name=settings.AWS_REGION,
        aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
        aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY,
    )
    s3 = session.resource("s3")

    return s3.Bucket(settings.AWS_S3_BUCKET)


class Defaults(object):
    CACHE_HEADER = str("max-age=5")
    ROOT_PATH = "elections"
    ACL = "public-read"


defaults = Defaults


class StorageService(S3Boto3Storage):
    bucket_name = settings.AWS_S3_BUCKET
    access_key = settings.AWS_ACCESS_KEY_ID
    secret_key = settings.AWS_SECRET_ACCESS_KEY
    file_overwrite = True
    querystring_auth = False
    object_parameters = {"CacheControl": "max-age=86400", "ACL": "public-read"}
    custom_domain = settings.CLOUDFRONT_ALTERNATE_DOMAIN
    location = settings.S3_UPLOAD_ROOT



from .get_field_type import get_field_type
from dateutil import parser
from datetime import datetime
from django.shortcuts import get_object_or_404


class Processor:
    @staticmethod
    def process_boolean(datum, field, model):
        if datum.lower() == "false":
            return False
        else:
            return bool(datum)

    @staticmethod
    def process_date(datum, field_name, model):
        if isinstance(datum, datetime):
            return datum
        else:
            return parser.parse(datum)

    @staticmethod
    def process_foreign(datum, field_name, model):
        related_model = [
            field
            for field in model._meta.get_fields()
            if field.name == field_name
        ][0].related_model
        return get_object_or_404(related_model, pk=datum)

    @staticmethod
    def process_string(datum, field_name, model):
        return datum

    @staticmethod
    def process_float(datum, field_name, model):
        return float(datum)

    @staticmethod
    def process_integer(datum, field_name, model):
        return int(datum)


def process_input_data(model, data):
    fields = {}

    for field in model._meta.get_fields():
        field_type = get_field_type(field)
        field_type = field_type if field_type else "unsupported"
        fields[field.name] = field_type

    for row in data:
        for field_name, datum in row.items():
            field_type = fields[field_name]
            process_func = getattr(
                Processor,
                "process_{}".format(field_type),
                lambda datum, field, model: datum,
            )
            row[field_name] = process_func(datum, field_name, model)

    return data



from datawok.conf import settings
from datawok.generators.views import (
    generate_upload_csv_file_view,
    generate_publish_view,
    generate_detail_view,
)
from datawok.models import datawok_models
from datawok.serializers import datawok_serializers
from datawok.utils.importers import import_class
from datawok.constants import DETAIL_VIEW, UPLOAD_CSV_FILE_VIEW, PUBLISH_VIEW

upload_csv_file_views = []
publish_views = []
detail_views = []

for i, model in enumerate(datawok_models):
    module_paths = settings.MODELS[i]

    # CSV Upload File Views
    upload_csv_file_view = (
        import_class(module_paths[UPLOAD_CSV_FILE_VIEW])
        if isinstance(module_paths, dict)
        and UPLOAD_CSV_FILE_VIEW in module_paths
        else generate_upload_csv_file_view(model)
    )
    upload_csv_file_views.append(upload_csv_file_view)

    # Publish Views
    publish_view = (
        import_class(module_paths[PUBLISH_VIEW])
        if isinstance(module_paths, dict) and PUBLISH_VIEW in module_paths
        else generate_publish_view(model)
    )
    publish_views.append(publish_view)

    # Detail Views
    detail_view = (
        import_class(module_paths[DETAIL_VIEW])
        if isinstance(module_paths, dict) and DETAIL_VIEW in module_paths
        else generate_detail_view(model, datawok_serializers[i])
    )
    detail_views.append(detail_view)



from rest_framework import viewsets
from rest_framework.response import Response
from django.contrib.auth.models import User
from datawok.utils.api_auth import TokenAPIAuthentication
from datawok.models import Query, Meta
from datawok.serializers import QuerySerializer


class QueryViewset(viewsets.ModelViewSet):
    authentication_classes = (TokenAPIAuthentication,)
    serializer_class = QuerySerializer
    permission_classes = []
    pagination_class = None
    queryset = Query.objects.all()

    def create(self, request):
        data = request.data.copy()

        try:
            data["datawok_model"] = Meta.objects.get(
                datawok_model__model=data["datawok_model"]
            ).datawok_model
        except Meta.DoesNotExist:
            return Response(
                "No Meta instance for model: {}.".format(
                    data["datawok_model"]
                ),
                404,
            )

        try:
            data["creator"] = User.objects.get(username=data["creator"])
        except Meta.DoesNotExist:
            return Response(
                "No user found named: {}.".format(data["creator"]), 404
            )

        q = Query(**data)
        q.save()

        return Response("OK", 200)



from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import Q

from .get_models import models as datawok_models


class Query(models.Model):
    """
    Meta information about your other models.
    """

    datawok_model = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        limit_choices_to=Q(
            model__in=[model.__name__.lower() for model in datawok_models]
        ),
    )

    creator = models.ForeignKey(User, on_delete=models.PROTECT)

    code = models.CharField(max_length=1000, blank=True, null=True)
    title = models.CharField(max_length=500, unique=True)

    def __str__(self):
        return self.title



from datawok.constants import (
    STRING_FIELDS,
    DATE_FIELDS,
    BOOLEAN_FIELDS,
    FK_FIELDS,
    INTEGER_FIELDS,
    FLOAT_FIELDS,
)


def get_field_type(field):
    for field_type in STRING_FIELDS:
        if isinstance(field, field_type):
            return "string"

    for field_type in DATE_FIELDS:
        if isinstance(field, field_type):
            return "date"

    for field_type in BOOLEAN_FIELDS:
        if isinstance(field, field_type):
            return "boolean"

    for field_type in INTEGER_FIELDS:
        if isinstance(field, field_type):
            return "integer"

    for field_type in FLOAT_FIELDS:
        if isinstance(field, field_type):
            return "float"

    for field_type in FK_FIELDS:
        if isinstance(field, field_type):
            return "foreign"

    return None



from django.db import models

from datawok.mixins import DatawokModelMixin
from postgres_copy import CopyManager


class DatawokModel(models.Model):
    objects = CopyManager()

    class Meta(DatawokModelMixin.Meta):
        abstract = True



from rest_framework import serializers
from datawok.models import Query


class QuerySerializer(serializers.ModelSerializer):
    datawok_model = serializers.SerializerMethodField()
    creator = serializers.SerializerMethodField()

    def get_datawok_model(self, obj):
        return str(obj.datawok_model)

    def get_creator(self, obj):
        return {"id": obj.creator.pk, "title": str(obj.creator)}

    class Meta:
        model = Query
        fields = "__all__"



import os

from setuptools import find_packages, setup

import datawok

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name=datawok.__package__,
    version=datawok.__version__,
    description="",
    url="https://github.com/The-Politico/django-datawok",
    author="POLITICO interactive news",
    author_email="interactives@politico.com",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Django",
        "Framework :: Django :: 2.0",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
    ],
    keywords="",
    packages=find_packages(exclude=["docs", "tests", "example"]),
    include_package_data=True,
    install_requires=[
        "python-dateutil",
        "psycopg2",
        "boto3",
        "django-postgres-copy",
    ],
    extras_require={"test": ["pytest"]},
)



from rest_framework import serializers


def generate_serializer(model):
    """
    Dynamically generates a viewset for a given model.
    """
    name = "{}Serializer".format(model.__name__.title())

    meta = type("Meta", (), {"model": model, "fields": "__all__"})

    return type(name, (serializers.ModelSerializer,), {"Meta": meta})



# flake8: noqa
from .get_serializers import (
    serializers as datawok_serializers,
    publish_serializers as datawok_publish_serializers,
)
from .meta import MetaSerializer
from .query import QuerySerializer



from django.db import models


class MetaCategory(models.Model):
    """
    Group models together.
    """

    slug = models.SlugField(max_length=50, unique=True)
    title = models.CharField(max_length=50, blank=True, null=True)

    def __str__(self):
        return self.title if self.title else self.slug



import json
from django.views.generic import TemplateView
from django.urls import reverse_lazy
from django.core.serializers.json import DjangoJSONEncoder
from datawok.conf import settings as app_settings
from datawok.utils.auth import secure


@secure
class Base(TemplateView):
    @staticmethod
    def prep_data_for_injection(queryset, serializer=None, many=False):
        if serializer:
            data = serializer(queryset, many=many).data
        else:
            data = queryset

        return json.dumps(json.dumps(data, cls=DjangoJSONEncoder))

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["user"] = self.request.user
        context["API_TOKEN"] = app_settings.SECRET_KEY
        context["AWS_PUBLIC_ROOT"] = app_settings.AWS_PUBLIC_ROOT
        context["APP_ROOT"] = app_settings.APP_ROOT
        context["API_ROOT"] = reverse_lazy("api-root")

        return context



from datawok.conf import settings
from datawok.generators.viewsets import generate_model_viewset
from datawok.models import datawok_models
from datawok.serializers import datawok_serializers
from datawok.utils.importers import import_class
from datawok.constants import VIEWSET

viewsets = []

for i, model in enumerate(datawok_models):
    module_paths = settings.MODELS[i]

    viewset = (
        import_class(module_paths[VIEWSET])
        if isinstance(module_paths, dict) and VIEWSET in module_paths
        else generate_model_viewset(model, datawok_serializers[i])
    )

    viewsets.append(viewset)



from datawok.conf import settings
from datawok.generators.serializers import generate_serializer
from datawok.models import datawok_models
from datawok.utils.importers import import_class
from datawok.constants import SERIALIZER, PUBLISH_SERIALIZER

serializers = []
publish_serializers = []

for i, model in enumerate(datawok_models):
    module_paths = settings.MODELS[i]

    serializer = (
        import_class(module_paths[SERIALIZER])
        if isinstance(module_paths, dict) and SERIALIZER in module_paths
        else generate_serializer(model)
    )

    serializers.append(serializer)

    publish_serializer = (
        import_class(module_paths[PUBLISH_SERIALIZER])
        if isinstance(module_paths, dict)
        and PUBLISH_SERIALIZER in module_paths
        else generate_serializer(model)
    )
    publish_serializers.append(publish_serializer)



"""
Use this file to configure pluggable app settings and resolve defaults
with any overrides set in project settings.
"""

from django.conf import settings as project_settings


class Settings:
    pass


Settings.AUTH_DECORATOR = getattr(
    project_settings,
    "DATAWOK_AUTH_DECORATOR",
    "django.contrib.auth.decorators.login_required",
)

Settings.SECRET_KEY = getattr(
    project_settings, "DATAWOK_SECRET_KEY", "a-bad-secret-key"
)

Settings.AWS_ACCESS_KEY_ID = getattr(
    project_settings, "DATAWOK_AWS_ACCESS_KEY_ID", None
)

Settings.AWS_SECRET_ACCESS_KEY = getattr(
    project_settings, "DATAWOK_AWS_SECRET_ACCESS_KEY", None
)

Settings.AWS_REGION = getattr(project_settings, "DATAWOK_AWS_REGION", None)

Settings.AWS_S3_BUCKET = getattr(
    project_settings, "DATAWOK_AWS_S3_BUCKET", None
)

Settings.CLOUDFRONT_ALTERNATE_DOMAIN = getattr(
    project_settings, "DATAWOK_CLOUDFRONT_ALTERNATE_DOMAIN", None
)

Settings.AWS_PUBLIC_ROOT = getattr(
    project_settings,
    "AWS_PUBLIC_ROOT",
    Settings.CLOUDFRONT_ALTERNATE_DOMAIN
    if Settings.CLOUDFRONT_ALTERNATE_DOMAIN
    else "http://s3.amazonaws.com/{}".format(Settings.AWS_S3_BUCKET),
)

Settings.S3_UPLOAD_ROOT = getattr(
    project_settings, "DATAWOK_S3_UPLOAD_ROOT", "uploads/datawok"
)

Settings.MODELS = getattr(project_settings, "DATAWOK_MODELS", [])

Settings.QUERY_TIMEOUT = getattr(
    project_settings, "DATAWOK_QUERY_TIMEOUT", 2000
)

Settings.QUERY_LIMIT = getattr(project_settings, "DATAWOK_QUERY_TIMEOUT", 1000)

Settings.DATABASE = getattr(project_settings, "DATAWOK_DATABASE", "default")

Settings.CSV_FILE_ENCODING = getattr(
    project_settings, "DATAWOK_CSV_FILE_ENCODING", "utf-8-sig"
)

Settings.APP_ROOT = getattr(project_settings, "DATAWOK_APP_ROOT", "")

settings = Settings
